@@ -3,9 +3,9 @@
 import os
 import sys,time,re
 import webbrowser
-from PyQt4 import QtGui,QtCore
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from PySide import QtGui,QtCore
+from PySide.QtCore import *
+from PySide.QtGui import *
 from pyRadio_ui import *
 from pyRadio_ui_portrait import *
 from radioplayer import RadioPlayer
@@ -35,8 +35,10 @@ class Config:
 		self.lastplayed = ""
 		self.format = "mp3"
 		self.rotation = "automatic"
+		self.protocol_version = "0"
+		self.proxy = ""
 		#I loop through possible configuration options in this array
-		self.config_opts = ['user','password','format','rotation']
+		self.config_opts = ['user','password','format','rotation','proxy','protocol_version']
 
 	def write(self): #write the configuration files
 		if not os.path.exists(os.path.expanduser("~")+'/.config'):
@@ -44,7 +46,9 @@ class Config:
 
 		if not os.path.exists(os.path.expanduser("~")+'/.config/pyradio'):
 			os.mkdir(os.path.expanduser("~")+'/.config/pyradio')
-		newfile = "user="+self.user+"\npassword="+self.password+"\n"+"format="+self.format+"\n"+"rotation="+self.rotation+"\n"
+		newfile = "user="+self.user+"\npassword="+self.password+"\n"+"format="+self.format+"\n"
+		newfile += "rotation="+self.rotation+"\n"+"protocol_version="+self.protocol_version+"\n"
+		newfile += "proxy="+self.proxy+"\n"
 		f = open(os.path.expanduser("~")+'/.config/pyradio/config', 'w')
 		f.write(newfile)
 		f.close()
@@ -55,22 +59,22 @@ class Config:
 			for line in f:
 				if re.match("ActiveTextColor\s*=",line):
 					self.color = re.match("ActiveTextColor\s*=(.+)",line).group(1).lstrip().rstrip()
-				
+
 			if self.color == "":
 				self.color = "rgb(255, 255, 255)"
 
 		else: #default to white
 			self.color = "rgb(255, 255, 255)"
-		
+
 		#open main configuration file
 		if os.path.exists(os.path.expanduser("~")+'/.config/pyradio/config'):
 			f = open(os.path.expanduser("~")+'/.config/pyradio/config', 'r')
 			for line in f:
 				line = line.replace(" ", "")
-				
+
 				if re.match("^#.*",line) is not None: #skip comments
 					continue
-				
+
 				for opt in self.config_opts: #check each line for an option
 					if re.match("^"+opt+"=(.+)$",line) is not None:
 						setattr(self,opt,self.get_var(opt,line))
@@ -85,13 +89,13 @@ class Config:
 	def get_var(self,var,line):
 		r = re.match("^"+var+"=(.+)$",line)
 		return r.group(1)
-		
+
 class Panda(QtGui.QMainWindow): #main class
 	def __init__( self, parent=None ):
-		QtGui.QWidget.__init__(self, parent)
-		self.c = Config() 
+		QtGui.QMainWindow.__init__(self, parent)
+		self.c = Config()
 		self.c.load()
-		self.s = snooze ( 0 )
+		self.s = snooze( 0 )
 		self.song = 0
 		self.stimer = QTimer()
 		self.manualPause = False #track manual pausing
@@ -104,33 +108,36 @@ class Panda(QtGui.QMainWindow): #main class
 		self.player = RadioPlayer()
 		self.stimer.singleShot(50, self.player.start)
 		self.os = self.player.get_product()
-		if self.os != "RX-51": #check for N900 
+		if self.os != "RX-51": #check for N900
 			self.ui.albumArt.setStyleSheet("background-color: rgb(255, 255, 255);")
 		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Pandora is loading... </font></td><tr></table>" )
 		QtGui.QApplication.processEvents()
 		self.player.format = self.c.format
+		self.player.protocol_version = int(self.c.protocol_version)
+		self.player.proxy = self.c.proxy
 		if not self.c.lastplayed == "": #auto-play last station
 			self.autoPlay = self.c.lastplayed
 		else:
-			self.autoPlay = 0 
+			self.autoPlay = 0
 		self.connect_signals() #called every time orientation changes to connect slots
 		self.stimer.singleShot(100, self.auth)
 		self.doFM(True)
-		try : #test for rotation and set auto-rotation on
-			self.connect(QApplication.desktop(), SIGNAL("resized(int)"), self.orientationChanged );
-			if self.c.rotation == 'automatic':
-				self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
-				screenGeometry=QApplication.desktop().screenGeometry();
-				if screenGeometry.width() > screenGeometry.height():
-					self.portrait = False
-				else:
-					self.portrait = True
-				print" Auto rotation on"
-			elif self.c.rotation == 'portrait':
-				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-			elif self.c.rotation == 'landscape':
-				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-		except:print "could not autorotate on"
+		try: #test for rotation and set auto-rotation on
+		    self.connect(QApplication.desktop(), SIGNAL("resized(int)"), self.orientationChanged );
+		    if self.c.rotation == 'automatic':
+		        self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
+		        screenGeometry=QApplication.desktop().screenGeometry();
+		        if screenGeometry.width() > screenGeometry.height():
+		            self.portrait = False
+		        else:
+		            self.portrait = True
+		            print "Auto rotation on"
+		    elif self.c.rotation == 'portrait':
+		        self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
+		    elif self.c.rotation == 'landscape':
+		        self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
+		except:
+		    print "could not autorotate on",
 
 #Radio Functions
 	def doPlayPause( self ): #Play if stopped, pause if playing
@@ -158,7 +165,7 @@ class Panda(QtGui.QMainWindow): #main class
 		"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
 		self.song[1]["rating"] = 1
 		self.stimer.singleShot(100, self.loveCall)
-		
+
 	def loveCall( self ):
 		if not self.player.loveHate( self.song, True ):
 			self.ui.btnLove.setStyleSheet("background-color: rgb(100, 100, 100);\n"
@@ -195,7 +202,7 @@ class Panda(QtGui.QMainWindow): #main class
 			self.c.user,ok = QtGui.QInputDialog.getText(self,"Username", "Pandora Username:", QLineEdit.Normal)
 		if not ok: #Username was cancelled.. they must be done.
 			self.closeEvent()
-		if self.c.password == "": 
+		if self.c.password == "":
 			self.c.password,ok = QtGui.QInputDialog.getText(self,"Password", "Pandora Password:", QLineEdit.Password)
 		if not ok: #Password was cancelled.. they must be done.
 			self.closeEvent()
@@ -207,13 +214,15 @@ class Panda(QtGui.QMainWindow): #main class
 			self.c.password = ""
 			self.auth( count )
 		else:
+			if str(self.player.pandora.protocol_version) != self.c.protocol_version:
+			    self.c.protocol_version = str(self.player.pandora.protocol_version)
 			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication successful.  Select a station from list to the right.</font></td><tr></table>" )
 			QtGui.QApplication.processEvents()
 			self.c.write()
 			self.getStations()
 
 	def playStation( self ): #Play currently selected station
-		stationId = self.stations[self.ui.listStations.currentIndex()]['stationId']	
+		stationId = self.stations[self.ui.listStations.currentIndex()]['stationId']
 		self.curStation = stationId
 		self.player.playlist = []
 		self.player.getMoreSongs( self.curStation, self.c.format )
@@ -223,7 +232,7 @@ class Panda(QtGui.QMainWindow): #main class
 			f.write(self.curStation)
 			f.close()
 
-	def getStations( self ): #Get Station List	
+	def getStations( self ): #Get Station List
 		self.ui.listStations.clear()
 		self.stations = self.player.getStations()
 		for s in self.stations: #Stations need to be scanned for QuickMix seperately
@@ -241,11 +250,11 @@ class Panda(QtGui.QMainWindow): #main class
 			self.ui.listStations.addItem(sName)
 			if self.autoPlay>0 and s["stationId"] == self.autoPlay:
 				self.autoPlay = 0
-				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
+				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)
 				self.playStation()
 
 			if self.curStation>0 and s["stationId"] == self.curStation:
-				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
+				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)
 
 	def setUserPass( self ): #Reset username/password and start authenticating
 		self.c.user = ""
@@ -266,7 +275,7 @@ class Panda(QtGui.QMainWindow): #main class
 		else:
 			self.player.getMoreSongs(self.curStation, self.c.format)
 			self.stimer.singleShot(100, self.playNextSong)
-	
+
 	def stop( self ): #Stop Playing, not currently implemented
 		print "Temp"
 
@@ -296,13 +305,13 @@ class Panda(QtGui.QMainWindow): #main class
 
 	def nextUp(self): #change button color
 		self.ui.btnNext.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")
-	
+
 	def hateDown(self): #change button color
 		self.ui.btnHate.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
 
 	def hateUp(self): #change button color
 		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
-	
+
 	def setDuration( self, pos, total ):
 		if (total-pos) < total:
 			position = "%02d:%02d" % divmod(total - pos, 60) + " / " + "%02d:%02d" % divmod(total, 60)
@@ -325,21 +334,21 @@ class Panda(QtGui.QMainWindow): #main class
 
 			state = fmtx_iface.Get("com.nokia.FMTx.Device", "state")
 			if state == 'disabled':
-				if not initial: 
+				if not initial:
 					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 					fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'enabled', variant_level=1))
 				else:
 					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 			else:
-				if not initial: 
+				if not initial:
 					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 					fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'disabled', variant_level=1))
 				else:
 					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
-		except: 
+		except:
 			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Set Max Volume", None, QtGui.QApplication.UnicodeUTF8))
 			if not initial:
-				self.player.setVolume( 100 ) 
+				self.player.setVolume( 100 )
 
 #Orientation Control
 
@@ -544,7 +553,7 @@ class Panda(QtGui.QMainWindow): #main class
 		self.d.show()
 
 	def donate( self, who=None ): #Open browser and close Donation window
-		if who == 'radio':		
+		if who == 'radio':
 			webbrowser.open_new('https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=BWX2M54JBJ2T2')
 		elif who == 'pandora':
 			webbrowser.open_new('http://www.pandora.com/pandora_one')
@@ -560,6 +569,7 @@ class Panda(QtGui.QMainWindow): #main class
 		hLayout3 = QHBoxLayout()
 		hLayout4 = QHBoxLayout()
 		hLayout5 = QHBoxLayout()
+		hLayout6 = QHBoxLayout()
 		self.settings['lblUsername'] = QLabel("Username:")
 		self.settings['lineUser'] = QLineEdit()
 		self.settings['lblUsername'].setBuddy(self.settings['lineUser'])
@@ -569,6 +579,10 @@ class Panda(QtGui.QMainWindow): #main class
 		self.settings['lblPass'].setBuddy(self.settings['linePass'])
 		self.settings['linePass'].setText(self.c.password)
 		self.settings['linePass'].setEchoMode(QLineEdit.Password)
+		self.settings['lblProxy'] = QLabel("Proxy Server:")
+		self.settings['lineProxy'] = QLineEdit()
+		self.settings['lblProxy'].setBuddy(self.settings['lineProxy'])
+		self.settings['lineProxy'].setText(self.c.proxy)
 		self.settings['btnGroup'] = QButtonGroup()
 		self.settings['radioHigh'] = QRadioButton("High (Pandora One)")
 		self.settings['radioMed'] = QRadioButton("Medium (MP3)")
@@ -603,6 +617,8 @@ class Panda(QtGui.QMainWindow): #main class
 		hLayout.addWidget(self.settings['lineUser'])
 		hLayout2.addWidget(self.settings['lblPass'])
 		hLayout2.addWidget(self.settings['linePass'])
+		hLayout6.addWidget(self.settings['lblProxy'])
+		hLayout6.addWidget(self.settings['lineProxy'])
 		hLayout3.addWidget(self.settings['radioHigh'])
 		hLayout3.addWidget(self.settings['radioMed'])
 		hLayout3.addWidget(self.settings['radioLow'])
@@ -615,6 +631,7 @@ class Panda(QtGui.QMainWindow): #main class
 		b.clicked.connect(self.d.close)
 		vbox.addLayout(hLayout)
 		vbox.addLayout(hLayout2)
+		vbox.addLayout(hLayout6)
 		vbox.addLayout(hLayout3)
 		if self.player.os == "RX-51":
 			vbox.addLayout(hLayout4)
@@ -630,6 +647,10 @@ class Panda(QtGui.QMainWindow): #main class
 			self.c.password = self.settings['linePass'].text()
 			self.auth()
 			changed = True
+		if self.c.proxy != self.settings['lineProxy'].text():
+			self.c.proxy = self.settings['lineProxy'].text()
+			self.player.pandora.set_proxy(self.c.proxy)
+			changed = True
 		if self.settings['radioHigh'].isChecked():
 			format = 'mp3-hifi'
 		elif self.settings['radioMed'].isChecked():
@@ -665,8 +686,8 @@ class Panda(QtGui.QMainWindow): #main class
 #Main Method
 
 if __name__ == '__main__':
-	if not ( os.path.exists( os.path.dirname(sys.argv[0])+"/libpiano/crypt_key_input.h" ) \
-			and os.path.exists( os.path.dirname(sys.argv[0])+"/libpiano/crypt_key_output.h" ) ):
+	if not ( os.path.exists( sys.path[0]+"/libpiano/crypt_key_input.h" ) \
+			and os.path.exists( sys.path[0]+"/libpiano/crypt_key_output.h" ) ):
 		ok = QtGui.QMessageBox.warning(None, "Pandora", "Missing encription key files." )
 	else:
 		app = QtGui.QApplication(sys.argv)
